class AgentModel:
    """
    OpenAI GPT Model Enum
    """
    GPT_4 = "gpt-4"
    GPT_4O = "gpt-4o"
    GPT_4O_MINI = "gpt-4o-mini"
    O1_MINI = "o1-mini"
    O1_PREVIEW = "o1-preview"
    CHATGPT_4O_LATEST = "chatgpt-4o-latest"
    LLAMA_3_1_405B = "llama-3.1-405"
    DEEPSEEK_R1_Distill_Qwen_14B = "DeepSeek-R1-Distill-Qwen-14B"
    AL_1S_CTF_VER = "al-1s-ctf-ver"
    QWQ_32B = "QwQ-32B"
    DEEPSEEK_R1 = "deepseek-r1"
    HIVE = "hive"
